; Displaying things on the Background Layer
;==============================================================================;
	.include "pce.inc"      ; hardware defines
	.include "ram_user.inc" ; user ram

;==============================================================================;
; some quick defines
DEFAULT_SATB_LOCATION = $7F00

DEFAULT_SCREEN_WIDTH  = 256
DEFAULT_SCREEN_HEIGHT = 240

DEFAULT_BAT_WIDTH  = 32
DEFAULT_BAT_HEIGHT = 32

;==============================================================================;
; Vector definitions (starting at $FFFA)
;==============================================================================;
	.segment "VECTORS"
	.addr fakeIRQ ; IRQ2
	.addr fakeIRQ ; IRQ1 (xxx: this is VDC-controlled and should be filled in)
	.addr Timer   ; Timer
	.addr fakeIRQ ; NMI
	.addr Reset   ; Reset

;==============================================================================;
; HuCard bank $00
;==============================================================================;
; This bank is loaded at $E000-$FFFF on system boot.

	.code

Reset:
	sei ; disable interrupts
	csh ; high speed mode
	cld ; clear decimal flag

	; set up stack
	ldx #$FF
	txs

	txa
	tam0 ; hardware bank $FF in mpr0
	lda #$F8
	tam1 ; ram bank $F8 in mpr1
	; physical bank $00 is in mpr7 by default

	; clear all ram (unsure if clearing the stack area is a good idea)
	stz $2000
	tii $2000,$2001,$1FFF

	jsr psg_init
	jsr vdc_init
	jsr vce_init

	jmp main

;==============================================================================;
; fakeIRQ
; Dummy interrupt vector used for NMI, IRQ1 (VDC), and IRQ2 (EXT).

.proc fakeIRQ
	rti
.endproc

;==============================================================================;
; Timer
; Dummy timer vector that acknowledges the timer interrupt.

.proc Timer
	stz IRQ_STATUS
	rti
.endproc

;==============================================================================;
; psg_init
; Initializes the PSG hardware.

.proc psg_init
	stz PSG_GLOBALVOL
	stz PSG_LFOCONTROL

	; reset all channels
	lda #5 ; (6-1)
@psg_init_clearloop:
	sta PSG_CHANSELECT
	stz PSG_CHANCTRL
	stz PSG_CHANPAN
	dec a
	bpl @psg_init_clearloop

	; disable noise on last two channels
	lda #4
	sta PSG_CHANSELECT
	stz PSG_NOISE
	inc a
	sta PSG_CHANSELECT
	stz PSG_NOISE

	rts
.endproc

;==============================================================================;
; vce_init
; Performs VCE initialization.

.proc vce_init
	; the dot clock should be based on the horizontal resolution:
	; < 267  = 5MHz
	; < 356  = 7MHz
	; >= 356 = 10MHz

	; default setting: 5MHz (since our default resolution is 256x240)
	lda #VCE_DOTCLOCK_5MHZ|VCE_BLUR_ON|VCE_MODE_COLOR
	sta VCE_CONTROL

	;-- set default background colors --;
	; background palette set 0, color 0
	stz VCE_ADDR_LO
	stz VCE_ADDR_HI

	; blue BG
	ldx #<$00C7
	ldy #>$00C7
	stx VCE_DATA_LO
	sty VCE_DATA_HI

	; sprite palette set 0, color 0
	ldx #<VCE_SPRPAL_START
	ldy #>VCE_SPRPAL_START
	stx VCE_ADDR_LO
	sty VCE_ADDR_HI
	; black BG
	stz VCE_DATA_LO
	stz VCE_DATA_HI

	rts
.endproc

;==============================================================================;
; vdc_init
; Performs VDC initialization.

.proc vdc_init
	; initialize CR (display off, disable all interrupts)
	st0 #VDCREG_CR
	st1 #0
	st2 #0

	; clear RCR
	st0 #VDCREG_RCR
	st1 #0
	st2 #0

	; reset BG X scroll
	st0 #VDCREG_BXR
	st1 #0
	st2 #0

	; reset BG Y scroll
	st0 #VDCREG_BYR
	st1 #0
	st2 #0

	;-- set up a 256x240 screen --;

	; VDC for 256x
	; HDS $02 \ _ HSR/$0A
	; HSW $02 /
	; HDE $04 \ _ HDR/$0B
	; HDW $1F /
	; CLK 0

	; set MWR/memory width register
	st0 #VDCREG_MWR
	; the value we want... (primarily dot width/clock 0)
	; FEDCBA9876543210
	; 0000000000000000
	; with a 32x32 BAT
	; xxxxxxxxxxHWWxxx
	; W=00, H=0
	st1 #0
	st2 #0

	; HPR/HSR
	st0 #VDCREG_HPR
	; HPR contains HDS and HSW
	; FEDCBA9876543210
	; 0|_____|000|___|
	;     |        |
	;     |        +--- HSW: %00010
	;     +------------ HDS: %0000010
	; $0202
	st1 #<$0202
	st2 #>$0202

	; HDR/HDR
	st0 #VDCREG_HDR
	; HDR contains HDE and HDW
	; FEDCBA9876543210
	; 0|_____|0|_____|
	;     |       |
	;     |       +---- HDW: %0011111
	;     +------------ HDE: %0000100
	; $041D
	st1 #<$041F
	st2 #>$041F

	;VDC for 240y
	; VSW $02 \
	; VDS $0C /
	; VDW $00EF
	; VCR $04

	; VSR
	st0 #VDCREG_VSR
	; VSR contains VDS and VSW
	; FEDCBA9876543210
	; |______|0|_____|
	;     |       |
	;     |       +---- VSW: %0000010
	;     +------------ VDS: %00001100
	; $0C02
	st1 #<$0C02
	st2 #>$0C02

	; VDR
	st0 #VDCREG_VDR
	st1 #<$00EF
	st2 #>$00EF

	; VCR
	st0 #VDCREG_VCR
	st1 #<$0004
	st2 #>$0004

	; DCR
	;st0 #VDCREG_DCR

	; SATB at $7F00
	st0 #VDCREG_SATB
	st1 #<DEFAULT_SATB_LOCATION
	st2 #>DEFAULT_SATB_LOCATION

	rts
.endproc

;==============================================================================;
; vram_clearBAT
; Clears the BAT using tile $0080.

; thanks to elmer for fixes

.proc vram_clearBAT
	; xxx: this routine is hardcoded to 32x32 BAT for the purpose of this example.

	; start at beginning of BAT
	st0 #VDCREG_MAWR
	stz a:VDC_DATA_LO
	stz a:VDC_DATA_HI

	; begin bat write
	st0 #VDCREG_VRWD
	lda #$80
	ldx #32
@rowLoop:
	ldy #32
@colLoop:
	sta a:VDC_DATA_LO
	stz a:VDC_DATA_HI
	dey
	bne @colLoop
	dex
	bne @rowLoop

	rts
.endproc

;==============================================================================;
main:
	;-- set up default colors for this demo --;
	; BG 0:1 - white
	ldx #<$0001
	ldy #>$0001
	stx VCE_ADDR_LO
	sty VCE_ADDR_HI
	ldx #<$01FF
	ldy #>$01FF
	stx VCE_DATA_LO
	sty VCE_DATA_HI

	;-- Load bank with background tiles into local memory --;
	lda #<.bank(gfx4BPP_font)
	tam2 ; background tiles now at $4000

	; set up source address for copying
	ldx #<gfx4BPP_font
	ldy #>gfx4BPP_font
	stx fontDataAddr
	sty fontDataAddr+1

	;-- Load background tiles into VRAM --;
	; First available location in VRAM should be $0800.
	st0 #VDCREG_MAWR
	st1 #<$0800
	st2 #>$0800

	; Set data load size (in words)
	; the graphics file is 4096 bytes, but only 3072 bytes are used.
	; divide by 2 for number of words in VRAM.
	;ldx #<($0C00>>1)
	;ldy #>($0C00>>1)

	ldx #<((gfx4BPP_font_end-gfx4BPP_font)>>1)
	ldy #>((gfx4BPP_font_end-gfx4BPP_font)>>1)
	stx font_loadSize
	sty font_loadSize+1

	; write data with VDCREG_VRWD
	st0 #VDCREG_VRWD

	; load loop goes here
	cly
	ldx font_loadSize
	beq @checkOuter
@fontLoadLoop:
	lda (fontDataAddr),y
	sta a:VDC_DATA_LO
	iny
	lda (fontDataAddr),y
	sta a:VDC_DATA_HI
	iny
	bne @checkInner

	inc fontDataAddr+1

@checkInner:
	dex
	bne @fontLoadLoop
	;jsr remap_data ; remap data if needed (e.g. if it crosses banks; not used here)

@checkOuter:
	dec font_loadSize+1
	bpl @fontLoadLoop

	;-- clear BAT using first blank tile ($0080) --;
	jsr vram_clearBAT

	;-- Write nt data to BAT --;
	; destination tile: x=4, y=4
	; (batwidth=32)*(y=4) + (x=4) = 132 ($84)
	st0 #VDCREG_MAWR
	st1 #<((32*2)+2)
	st2 #>((32*2)+2)

	; write some random tiles (ASCII $20 maps to tile $080)
	st0 #VDCREG_VRWD
	;st1 #<$00C1 ; a
	;st2 #>$00C1
	st1 #<('a'+$60)
	st2 #>('a'+$60)

	;st1 #<$00D3 ; s
	;st2 #>$00D3
	st1 #<('s'+$60)
	st2 #>('s'+$60)

	;st1 #<$00C4 ; d
	;st2 #>$00C4
	st1 #<('d'+$60)
	st2 #>('d'+$60)

	;st1 #<$00C6 ; f
	;st2 #>$00C6
	st1 #<('f'+$60)
	st2 #>('f'+$60)

	;-- re-enable interrupts --;
	lda #1 ; inhibit IRQ2, allow IRQ1 and Timer
	sta IRQ_DISABLE
	stz IRQ_STATUS
	cli

	;-- Turn on display --;
	st0 #VDCREG_CR
	st1 #<(VDC_CR_VBL|VDC_CR_SPR|VDC_CR_BG|VDC_CR_INC_1)
	st2 #>(VDC_CR_VBL|VDC_CR_SPR|VDC_CR_BG|VDC_CR_INC_1)

forever:
	bra forever

;==============================================================================;
; todo: eventually, this example code should be able to print this string
str_HelloWorld:
	.byte "Hello World",$00


;==============================================================================;
; HuCard bank $01
;==============================================================================;
; This bank will be loaded at $4000-$5FFF

	.segment "BANK01"

; 3KB of 4bpp (BG) font data
gfx4BPP_font:
	.incbin "font_bg.4bpp_bg"
gfx4BPP_font_end:

